(*
    Id3Tag - Property editor
    Copyright (c) 1998-2002,2003 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit Properties;

interface

uses
  Id3,
  SysUtils, Classes, QForms, QComCtrls, QStdCtrls, QGraphics, QExtCtrls,
  QControls;

type
  TformId3Properties = class(TForm)
    edtTitle: TEdit;
    lblTitle: TLabel;
    edtArtist: TEdit;
    lblArtist: TLabel;
    edtAlbum: TEdit;
    lblAlbum: TLabel;
    edtYear: TEdit;
    lblYear: TLabel;
    edtComment: TEdit;
    lblComment: TLabel;
    lblGenre: TLabel;
    cboGenre: TComboBox;
    btnOk: TButton;
    btnCancel: TButton;
    btnApply: TButton;
    bvlBeforeId3: TBevel;
    imgIcon: TImage;
    lblLocationTag: TLabel;
    lblLocation: TLabel;
    bvlAfterFilename: TBevel;
    lblSizeTag: TLabel;
    lblSize: TLabel;
    lblId3Tag: TLabel;
    lblId3: TLabel;
    imgId3v1: TImage;
    lblTrack: TLabel;
    edtFilename: TEdit;
    edtTrack: TSpinEdit;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure edtTitleChange(Sender: TObject);
    procedure edtArtistChange(Sender: TObject);
    procedure edtAlbumChange(Sender: TObject);
    procedure edtYearChange(Sender: TObject);
    procedure edtCommentChange(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure cboGenreChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure edtTitleEnter(Sender: TObject);
    procedure edtTitleKeyPress(Sender: TObject; var Key: Char);
    procedure edtTrackChanged(Sender: TObject; NewValue: Integer);
  private
    { Private declarations }
    FFilename: string;
    FInfo: TId3Info;
    procedure Changed;
    procedure DisableApply;
    procedure SetFilename( Value: string );
    procedure SetInfo( Value: TId3Info );
  public
    { Public declarations }
    property Filename: string read FFilename write SetFilename;
    property Info: TId3Info read FInfo write SetInfo;
  end;

var
  formId3Properties: TformId3Properties;

implementation

uses Main;

{$R *.dfm}

function CalcFileSize( Name: string ): integer;
var
  F: file;
begin
  try
    Assign( F, Name );
    FileMode := 0;
    Reset( F, 1 );
    Result := Filesize( F );
    Close( F );
  except
    Result := 0;
  end;
end;

procedure TformId3Properties.SetFilename( Value: string );
var
  Path: array[1..MAX_PATH] of char;
  Index: word;
  Size: integer;
begin
  FFilename := Value;
  if( Value = '' ) then begin
    edtFilename.Text := 'N/A (multiple files)';
    lblLocation.Caption := 'N/A';
    lblSize.Caption := 'N/A';
    imgIcon.Picture.Assign( imgId3v1.Picture );
    imgIcon.Visible := true;
    (* Disable title editing *)
    lblTitle.Enabled := false;
    edtTitle.Enabled := false;
    (* Disable track editing *)
    lblTrack.Enabled := false;
    edtTrack.Enabled := false;
  end
  else begin
    edtFilename.Text := ExtractFilename( Value );
    lblLocation.Caption := ExtractFilepath( Value );
    StrCopy( @Path, PChar(Value) );
    Index := 0;

    // For now, no icon selection on Linux...
    imgIcon.Picture.Assign( imgId3v1.Picture );
    imgIcon.Visible := true;

    Size := CalcFileSize( Value );
    lblSize.Caption := Format( '%.0nKB (%d bytes)', [Int(Size / 1024), Size] );
    (* Enable title editing *)
    lblTitle.Enabled := true;
    edtTitle.Enabled := true;
    (* Enable track editing *)
    lblTrack.Enabled := true;
    edtTrack.Enabled := true;
  end;
end;

procedure TformId3Properties.SetInfo( Value: TId3Info );
begin
  FInfo.Assign( Value );
  edtTitle.Text := FInfo.Title;
  edtArtist.Text := FInfo.Artist;
  edtAlbum.Text := FInfo.Album;
  edtYear.Text := FInfo.Year;
  edtComment.Text := FInfo.Comment;
  edtTrack.Value := FInfo.Track;
  cboGenre.ItemIndex := cboGenre.Items.IndexOfObject( TObject(FInfo.Genre) );
  if( ifHasTag in FInfo.Flags ) then
    lblId3.Caption := 'present'
  else
    lblId3.Caption := 'not present';
end;

procedure TformId3Properties.FormShow(Sender: TObject);
begin
  edtTitle.Hint := edtTitle.Text;
  if( not edtTitle.Enabled ) then begin
    edtArtist.SetFocus
  end
  else begin
    edtTitle.SetFocus;
    edtTitle.SelectAll;
  end;
  DisableApply;
end;

procedure TformId3Properties.FormCreate(Sender: TObject);
var
  I: integer;
begin
  cboGenre.Clear;
  for I:=0 to MaxGenre do
    cboGenre.Items.AddObject( GetGenreName(I), pointer(I) );
  cboGenre.Items.AddObject( '- none -', pointer(255) );
  FInfo := TId3Info.Create;
end;

procedure TformId3Properties.edtTitleChange(Sender: TObject);
begin
  Info.Title := edtTitle.Text;
  edtTitle.Color := clInfoBk;
  if( edtTitle.Enabled ) then
    Changed;
end;

procedure TformId3Properties.edtArtistChange(Sender: TObject);
begin
  Info.Artist := edtArtist.Text;
  edtArtist.Color := clInfoBk;
  Changed;
end;

procedure TformId3Properties.edtAlbumChange(Sender: TObject);
begin
  Info.Album := edtAlbum.Text;
  edtAlbum.Color := clInfoBk;
  Changed;
end;

procedure TformId3Properties.edtYearChange(Sender: TObject);
begin
  Info.Year := edtYear.Text;
  edtYear.Color := clInfoBk;
  Changed;
end;

procedure TformId3Properties.edtCommentChange(Sender: TObject);
begin
  Info.Comment := edtComment.Text;
  edtComment.Color := clInfoBk;
  Changed;
end;

procedure TformId3Properties.Changed;
begin
  btnApply.Enabled := true;
end;

procedure TformId3Properties.btnApplyClick(Sender: TObject);
begin
  FormMain.Apply( FInfo );
  FInfo.ClearChanged;
  DisableApply;
end;

procedure TformId3Properties.cboGenreChange(Sender: TObject);
begin
  if( cboGenre.ItemIndex >= 0 ) then
    Info.Genre := integer( cboGenre.Items.Objects[cboGenre.ItemIndex] )
  else
    Info.Genre := -1;
  cboGenre.Color := clInfoBk;
  Changed;
end;

procedure TformId3Properties.DisableApply;
begin
  edtTitle.Color := clWindow;
  edtArtist.Color := clWindow;
  edtAlbum.Color := clWindow;
  edtYear.Color := clWindow;
  edtTrack.Color := clWindow;
  cboGenre.Color := clWindow;
  edtComment.Color := clWindow;
  btnApply.Enabled := false;
end;

procedure TformId3Properties.FormDestroy(Sender: TObject);
begin
  FInfo.Free;
end;

procedure TformId3Properties.edtTitleEnter(Sender: TObject);
var
  E: TEdit;
begin
  E := Sender as TEdit;
  E.Hint := E.Text;
end;

procedure TformId3Properties.edtTitleKeyPress(Sender: TObject;
  var Key: Char);
var
  E: TEdit;
begin
  E := Sender as TEdit;
  if( Key = #27 ) then begin
    E.Text := E.Hint;
    E.SelStart := 0;
    E.SelLength := Length(E.Text);
    Key := #0;
  end;
end;

procedure TformId3Properties.edtTrackChanged(Sender: TObject; NewValue: Integer);
begin
  edtTrack.Color := clInfoBk;
  Info.Track := NewValue;
end;

end.
